/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * sandnet.h
 * Interface to libsandnet
 * Copyright (C) 2006 Simon Newton
 */

/** @file sandnet/sandnet.h */

#ifndef _defined_sandnet_h
#define _defined_sandnet_h

#include <stdint.h>
// order is important here for osx
#include <sys/types.h>
#include <sys/select.h>
#include <sandnet/common.h>

#ifdef __cplusplus
extern "C" {
#endif

// error codes
#define SANDNET_EOK  0
#define SANDNET_ENET -1      // network error
#define SANDNET_EMEM -2      // memory error
#define SANDNET_EARG -3      // argument error
#define SANDNET_ESTATE -4    // state error
#define SANDNET_EACTION -5    // invalid action


/*
 * port directions
 */
enum sandnet_port_mode_e {
  SANDNET_PORT_MODE_DISABLED,
  SANDNET_PORT_MODE_OUT,
  SANDNET_PORT_MODE_IN,
  SANDNET_PORT_MODE_MOUT,
  SANDNET_PORT_MODE_MIN
};

typedef enum sandnet_port_mode_e sandnet_port_mode_t;



/**
 * Enums for the application defined handlers
 */
typedef enum {
  SANDNET_RECV_HANDLER,
  SANDNET_AD_HANDLER,
  SANDNET_CTL_HANDLER,
  SANDNET_DMX_HANDLER,
  SANDNET_NAME_HANDLER,
  SANDNET_IDENTIFY_HANDLER,
  SANDNET_PROG_HANDLER,
  SANDNET_LED_HANDLER,
  SANDNET_DMX2_HANDLER,
  SANDNET_LAST_HANDLER,
} sandnet_handler_name_t;




/** The local SandNet node */
typedef void *sandnet_node;

extern char *sandnet_strerror();

// node control functions
extern sandnet_node sandnet_new(const char *ip, int verbose);
extern int sandnet_start(sandnet_node n);
extern int sandnet_read(sandnet_node n, int timeout);
extern int sandnet_stop(sandnet_node n);
int sandnet_destroy(sandnet_node vn);


// handler functions
extern int sandnet_set_dmx_handler(sandnet_node vn, int (*fh)(sandnet_node n, uint8_t group, uint8_t universe, int length, uint8_t *data,  void *d), void *data );
extern int sandnet_set_handler(sandnet_node vn, sandnet_handler_name_t handler, int (*fh)(sandnet_node n, void *pp, void *d), void* data);


// send functions
extern int sandnet_send_dmx(sandnet_node n, int port, int16_t length, const uint8_t *data);
extern int sandnet_raw_send_dmx(sandnet_node n, uint8_t port, uint8_t grp, uint8_t universe, int16_t length, const uint8_t *data);

extern int sandnet_send_ad(sandnet_node n);

// config functions
extern int sandnet_set_name(sandnet_node vn, const char *name);
extern int sandnet_set_port(sandnet_node vn, int port, sandnet_port_mode_t mode, uint8_t param1, uint8_t param2);

// misc
extern int sandnet_get_sd(sandnet_node n, int i);

#ifdef __cplusplus
}
#endif

#endif
